#!/usr/bin/perl

# Script to extacrt OCR information from FineReader FRF and add to the DJVU book
# Written by Alexey Semenoff [http://swaj.net]

use strict;
use Cwd;
use File::Copy;
no warnings;

use vars qw /
    $VERSION
    $PATH
    $OS
    $DJVU_BOOK
    $DDJVU
    $DJVUSED
    $EMPTY_TIFF
    $WORK_DIR
    $FORMAT
    /;

BEGIN {
    $|++;

    $VERSION = '0.5';
    if ( $^O =~ /^(ms)?(dos|win(32|nt)?)/i ) {
        $OS = 'WIN';
    }
    else {
        $OS = 'UNIX';
    }

    $WORK_DIR =  cwd();

    $PATH = $0;
    $PATH =~ s/^(.+[\/|\\]).+/$1/ ;

    $FORMAT = 'tiff'; # 'tiff' | 'pbm'

    if ($OS eq 'WIN') {
	$PATH =~ s|/|\\|g;
	$WORK_DIR =~ s|/|\\|g;
	$DDJVU = "$PATH\\DDJVUT.EXE";
	$DJVUSED = "$PATH\\DJVUSED.EXE";
	$EMPTY_TIFF = "$PATH\\EMPTY-200X300.$FORMAT";

    }
    else {
	$DDJVU = "$PATH/ddjvu";
	$DJVUSED = "$PATH/djvused";
	$EMPTY_TIFF = "$PATH/empty-200x300.$FORMAT";
    }


}


###
### Checks for binaries
###
die "No file '$EMPTY_TIFF' \n" unless (-f $EMPTY_TIFF);
die "No file '$DDJVU' \n" unless (-f $DDJVU);
die "No file '$DJVUSED' \n" unless (-f $DJVUSED);


###
### Splash etc
###
print <<EOS;
+---------------------------------------------------------------
| DJVU TO TIFF tool, version $VERSION started ($OS)
| This script was written by Alexey Semenoff [http://swaj.net]
| Work directory is: '$WORK_DIR'
| File format is: '$FORMAT' 
+---------------------------------------------------------------

Press <Enter> to continue...
EOS
<STDIN>;


###
### Which DJVU to use
###
my @djvus = glob ("*.djvu");
my $djvusn = scalar (@djvus);

die "No .DJVU files found!\n" unless $djvusn;
die "Too many .DJVU files found, only one should be here!\n" if ($djvusn != 1);

$DJVU_BOOK = $djvus[0];
my $pages = get_pages_tot();
print "Use DJVU book '$DJVU_BOOK', pages: $pages\n";


###
### Convert all pages
###
print "Converting pages\n";
for (my $page=1; $page <= $pages; $page++)
{
    my $file2 = sprintf ("page_%04.d.%s", $page, $FORMAT);
    print "Saving page $page --> '$file2'...\n";

    $_ = system ("$DDJVU -format=$FORMAT -page=$page -mode=mask $DJVU_BOOK $file2");

    if ($_) {
        print "copy TIFF from empty...";
        die "no file '$EMPTY_TIFF'" unless -f $EMPTY_TIFF;
	copy($EMPTY_TIFF, $file2);
    }

    die "page failed" unless -f $file2;
}

###
### WE are done, quitting
###
print "\n*** COMPLETED!\n";
exit 0;



###
### How many pages in DJVU
###
sub get_pages_tot {
    $_ = `$DJVUSED $DJVU_BOOK -e 'n' `;
    chomp;
    s|\D||g;
    return $_; 
}


###
### Simple dialog
###
sub ask_continue {
    print "\nWould you like to continue <Y/N> ?";
    $_ = <STDIN>;
    chomp;
    die "Aborted\n" if ($_ ne 'y' && $_ ne 'Y');
}


__END__
