#!/usr/bin/perl

# Script to extacrt OCR information from FineReader FRF and add to the DJVU book
# Written by Alexey Semenoff [http://swaj.net]

use Cwd;
use strict;
no warnings;

use vars qw /
    $VERSION
    $PATH
    $OS
    $DJVUSED
    $FRFGRAB
    $WORK_DIR
    $DJVU_BOOK
    $ERROR
    $TXT_FILE
    $TXT_FILE2
    $TXT_FILE3
    /;

BEGIN {

    $|++;
    $ERROR = 0;
    $TXT_FILE = 'ocr.txt';
    $TXT_FILE2 = 'ocr-lines-utf8.txt';
    $TXT_FILE3 = 'ocr-lines-utf8-test.txt';


    $VERSION = '0.5';
    if ( $^O =~ /^(ms)?(dos|win(32|nt)?)/i ) {
        $OS = 'WIN';
    }
    else {
        $OS = 'UNIX';
    }

    die "Current version works under WIN32 only!\n" if ($OS eq 'UNIX');

    $WORK_DIR =  cwd();

    $PATH = $0;
    $PATH =~ s/^(.+[\/|\\]).+/$1/ ;

    $PATH =~ s|/|\\|g;
    $WORK_DIR =~ s|/|\\|g;
    $DJVUSED = "$PATH\\DJVUSED.EXE";
    $FRFGRAB  = "$PATH\\FRFgrab.exe";

}

###
### print "\n test \n"; merge_words ('ocr.txt', 'ocr.utf8'); die;
###

###
### Splash etc
###
print <<EOS;
+---------------------------------------------------------------
| DJVU OCR tool, version $VERSION started ($OS)
| This script was written by Alexey Semenoff [http://swaj.net]
| Work directory is: '$WORK_DIR'
| RAW OCR file: '$TXT_FILE'
| UTF OCR (lines) file: '$TXT_FILE2'
+---------------------------------------------------------------

Press <Enter> to continue...
EOS
<STDIN>;


###
### Checks for binaries
###
die "No file '$DJVUSED' \n" unless (-f $DJVUSED);
die "No file '$FRFGRAB' \n" unless (-f $FRFGRAB);


###
### FRFs, how many etc
###
print "Counting FRF files\n";
my @frfs = glob ("*.frf");
my $frfsn = scalar ( @frfs );
die "No .FRF files found!\n" unless glob $frfsn;
print "FRF files here: $frfsn\n";


###
### Which DJVU to use
###
my @djvus = glob ("*.djvu");
my $djvusn = scalar (@djvus);

die "No .DJVU files found!\n" unless $djvusn;
die "Too many .DJVU files found, only one should be here!\n" if ($djvusn != 1);

$DJVU_BOOK = $djvus[0];
my $pages = get_pages_tot();
print "Use DJVU book '$DJVU_BOOK', pages: $pages\n";


###
### Number of pages mathes FRFs ?
###
if ($frfsn !=  $pages) {
    print "\n!!! WARN: Pages in DJVU: $pages, FRF files: $frfsn , possible wrong layout!\n";
    ask_continue();
}


###
### Test FRFs
###
print "Testing FRF...\n";
$ERROR = 0;
open (F, "$FRFGRAB -t -q *.frf |") or die;
while (<F>) {
    chomp;
    if ($_ ne q{}) {
	print "$_\n";
	$ERROR++;
    }
}
close F;

if ($ERROR) {
    print "\n!!! WARN: Testing FRF failed, some files can be unusable!\n";
    ask_continue();
}
else {
    print "\nTesting FRF OK!\n";
}


###
### Extracting OCR --> file.txt
###
print "Extracting OCR info...\n";
unlink $TXT_FILE;
system ("$FRFGRAB -g *.frf > $TXT_FILE");

die ("\nOCR file was not created\n") unless (-f $TXT_FILE);


###
### Merging words to lines, reducing spaces
###
print "\nMerging words, converting to utf8\n";
merge_words ($TXT_FILE, $TXT_FILE2);
die "Failed\n" unless (-f $TXT_FILE2);


###
### Including TXTz layer to DJVU
###
print "Including TXTz layer\n";
system ("$DJVUSED -f $TXT_FILE2 $DJVU_BOOK");


###
### Testing
###
print "Testing\n";
system ("$DJVUSED -e output-txt $DJVU_BOOK > $TXT_FILE3");
open (TF, "< $TXT_FILE3") or die "Cannot open file '$TXT_FILE3' :$!\n";
print "Counting text pages\n";

my $pt = 0;
while (<TF>) {
    chomp;
    $pt++ if (/^select\s+/);
}
close TF;

print "DJVU contains $pages pages, TXTz pages $pt\n";

if ($pt != $pages) {
    print "\n!!! WARN: Pages in DJVU: $pages, TXTz pages in document: $pt !\n";
    die;
}

###
### WE are done, quitting
###
print "\n*** COMPLETED!\n";
exit 0;


###
### words --> lines
###
sub merge_words {
    my ($infile, $outfile) = @_;
    die "No files specified" if (! $infile || ! $outfile);

    open (IF, "< $infile") or die "cannot open file '$infile':$!\n";
    open (OF, "> $outfile") or die "cannot create file '$outfile':$!\n";

    my $words_handling;
    my $line = q{};
    my $line_count=0;
    my $sline = q{};
    my $prev_line = q{};

    while (<IF>) {
	$line_count++;
	s|\r||is;
	chomp;
	$sline = $_;

	if ( ($line_count > 2) && ( /^\#\s*\-/ ) && ( $prev_line !~ /^$/ ) )
	  {
	    print "malformed OCR data, line $line_count, fixing...\n";
	    print OF ")\n\n.\n\n";
	  }

	$prev_line = $sline;

	if ( ! /\(\s*word/ ) # no ( word ... pattern
	{
	    print OF "$_\n"; # -> OF
	    $line = q{};
	    next;
	}


	if ( /\(\s*word.+\)\)\)?\s*$/ ) # )) or )))
	{
	    s|.+?\"(.+)\".*|$1|;
	    die "no match" if ($1 eq q{});
	    if ($line ne q{}) {
		$line .= ' ';
	    }

	    $line .= $1;

	    my $br = q{};
	    if ($sline =~/\)\)\)\s*$/) {
		$br = ')';
	    }
	    convert_line (\$line);
	    print OF '"', $line , '")', $br, "\n"; # -> OF
	    $line = q{};
	    next;

	}
	elsif ( /\(\s*word.+\)\s*$/ )
	{
	    s|.+?\"(.+)\".*|$1|;
	    die "no match" if ($1 eq q{});
	    if ($line ne q{}) {
		$line .= ' ';
	    }

	    $line .= $1;
	    next;
	}

	die "wrong state at line '$_'";

    }

    close IF;
    close OF;

}


###
### \123 --> utf8
###
sub convert_line {
    my $line_ref = $_[0];
    my ($s, $n, $v);

    while ($$line_ref =~ /\\(\d\d\d)/sg) {
	$s = $1;
	die "wrong line '$$line_ref'" if ($s eq q{});
	$v = oct ($1);
	$n = chr ($v);

	if (  $v >= 128 && $v < 255 ) { # only upper ascii, A-Z a-z by ffrgrab
	    $$line_ref =~ s|\\$s|$n|;
	}
    }

}


###
### How many pages in DJVU
###
sub get_pages_tot {
    $_ = `$DJVUSED $DJVU_BOOK -e 'n' `;
    chomp;
    s|\D||g;
    return $_; 
}


###
### Simple dialog
###
sub ask_continue {
    print "\nWould you like to continue <Y/N> ?";
    $_ = <STDIN>;
    chomp;
    die "Aborted\n" if ($_ ne 'y' && $_ ne 'Y');
}


__END__
